import requests
import json
import re
import os
import logging

from KicadModTree import *
from .schematic_handlers import *


template_lib_header = f"""\
(kicad_symbol_lib (version 05052022) (generator TousstNicolas/JLC2KiCad_lib)
"""

template_lib_footer = """
)"""


def create_schematic(
	schematic_component_uuid, footprint_name, datasheet_link, library_name, output_dir, component_id
):
	class kicad_schematic:
		drawing = ""
		pinNamesHide = ""
		pinNumbersHide = ""

	kicad_schematic = kicad_schematic()

	ComponentName = ""
	for component_uuid in schematic_component_uuid:

		response = requests.get(f"https://easyeda.com/api/components/{component_uuid}")
		if response.status_code == requests.codes.ok:
			data = json.loads(response.content.decode())
		else:
			logging.error(
				f"create_schematic error. Requests returned with error code {response.status_code}"
			)
			return ()

		schematic_shape = data["result"]["dataStr"]["shape"]
		symmbolic_prefix = data["result"]["packageDetail"]["dataStr"]["head"]["c_para"][
			"pre"
		].replace("?", "")
		component_title = (
			data["result"]["title"].replace("/", "_").replace(" ", "_").replace(".", "_")
		)

		if not ComponentName:
			ComponentName = component_title
			component_title += "_0"
		if (
			len(schematic_component_uuid) >= 2 and component_uuid == schematic_component_uuid[0]
		):
			continue

		filename = f"{output_dir}/Schematic/" + library_name + ".kicad_sym"

		logging.info(f"creating schematic {component_title} in {library_name}")

		kicad_schematic.drawing += f'''\n    (symbol "{component_title}_0"'''
		for line in schematic_shape:
			args = [i for i in line.split("~") if i]  # split and remove empty string in list
			model = args[0]
			logging.debug(args)
			if model not in handlers:
				logging.warning("Schematic : parsing model not in handler : " + model)
			else:
				handlers.get(model)(args[1:], kicad_schematic)
		kicad_schematic.drawing += """\n    )"""

	template_lib_component = f"""\
  (symbol "{ComponentName}" {kicad_schematic.pinNamesHide} {kicad_schematic.pinNumbersHide} (in_bom yes) (on_board yes)
    (property "Reference" "{symmbolic_prefix}" (id 0) (at 0 1.27 0)
      (effects (font (size 1.27 1.27)))
    )
    (property "Value" "{ComponentName}" (id 1) (at 0 -2.54 0)
      (effects (font (size 1.27 1.27)))
    )
    (property "Footprint" "{footprint_name}" (id 2) (at 0 -10.16 0)
      (effects (font (size 1.27 1.27) italic) hide)
    )
    (property "Datasheet" "{datasheet_link}" (id 3) (at -2.286 0.127 0)
      (effects (font (size 1.27 1.27)) (justify left) hide)
    )
    (property "LCSC" "{component_id}" (id 4) (at 0 0 0)
      (effects (font (size 1.27 1.27)))
    ){kicad_schematic.drawing}
  )"""

	if not os.path.exists(f"{output_dir}/Schematic"):
		os.makedirs(f"{output_dir}/Schematic")

	if os.path.exists(filename):
		update_library(library_name, ComponentName, template_lib_component, output_dir)
	else:
		with open(filename, "w") as f:
			logging.info(f"writing in {filename} file")
			f.write(template_lib_header)
			f.write(template_lib_footer)
		update_library(library_name, ComponentName, template_lib_component, output_dir)


def update_library(library_name, component_title, template_lib_component, output_dir):
	"""
	if component is already in library,
	the library will be updated,
	if not already present in library,
	the component will be added at the beginning
	"""

	with open(f"{output_dir}/Schematic/{library_name}.kicad_sym", "rb+") as lib_file:
		pattern = f'  \(symbol "{component_title}" \(pin_names (\n|.)*?\n  \)'
		file_content = lib_file.read().decode()

		if f'symbol "{component_title}" (pin_names' in file_content:
			# use regex to find the old component template in the file and replace it with the new one
			logging.info(f"found component already in {library_name}, updating {library_name}")
			sub = re.sub(
				pattern=pattern,
				repl=template_lib_component,
				string=file_content,
				flags=re.DOTALL,
				count=1,
			)
			lib_file.seek(0)
			# delete the file content and rewrite it
			lib_file.truncate()
			lib_file.write(sub.encode())
		else:
			# move before the library footer and write the component template
			lib_file.seek(-len(template_lib_footer), 2)
			lib_file.truncate()
			lib_file.write(template_lib_component.encode())
			lib_file.write(template_lib_footer.encode())



import sys
import os

from setuptools import setup, find_packages


long_description = open(os.path.join(sys.path[0], "README.md")).read()

setup(
	name="JLC2KiCadLib",
	description="JLC2KiCad_lib is a python script that generate a component library (schematic, footprint and 3D model) for KiCad from the JLCPCB/easyEDA library.",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/TousstNicolas/JLC2KiCad_lib",
	version="1.0.4",
	author="TousstNicolas",
	license="MIT",
	install_requires=["KicadModTree",],
	packages_dir={"JLC2KiCadLib": "JLC2KiCadLib"},
	packages=find_packages(exclude=[]),
	entry_points={"console_scripts": ["JLC2KiCadLib = JLC2KiCadLib.JLC2KiCadLib:main"]},
	classifiers=[
		"Programming Language :: Python :: 3.6",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
		"Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)",
	],
	python_requires=">=3.6",
)



import requests
import json
import logging
import argparse

from . import helper
from .footprint.footprint import create_footprint, get_footprint_info
from .schematic.schematic import create_schematic


def add_component(component_id, args):

	logging.info(f"creating library for component {component_id}")
	data = json.loads(
		requests.get(f"https://easyeda.com/api/products/{component_id}/svgs").content.decode()
	)

	if not data["success"]:
		logging.error(f"failed to get component uuid for {component_id}\nThe component # is probably wrong. Check a possible typo and that the component exists on easyEDA")
		return ()

	footprint_component_uuid = data["result"][-1]["component_uuid"]
	schematic_component_uuid = [i["component_uuid"] for i in data["result"][:-1]]

	if args.footprint_creation:
		footprint_name, datasheet_link = create_footprint(
			footprint_component_uuid=footprint_component_uuid,
			component_id=component_id,
			footprint_lib=args.footprint_lib,
			output_dir=args.output_dir,
			model_path_relative=args.model_path_relative,
		)
	else:
		_, datasheet_link, _ = get_footprint_info(component_id)
		footprint_name = ""

	if args.schematic_creation:
		create_schematic(
			schematic_component_uuid=schematic_component_uuid,
			footprint_name=footprint_name,
			datasheet_link=datasheet_link,
			library_name=args.schematic_lib,
			output_dir=args.output_dir,
			component_id=component_id,
		)


def main():
	parser = argparse.ArgumentParser(
		description="take a JLCPCB part # and create the according component's kicad's library",
		epilog="exemple use : \n	python3 JLC2KiCad_lib.py C1337258 C24112 -dir My_lib -schematic_lib My_Schematic_lib --no_footprint",
		formatter_class=argparse.RawDescriptionHelpFormatter,
	)

	parser.add_argument(
		"components",
		metavar="JLCPCB_part_#",
		type=str,
		nargs="+",
		help="list of JLCPCB part # from the components you want to create",
	)

	parser.add_argument(
		"-dir",
		dest="output_dir",
		type=str,
		default="JLC2KiCad_lib",
		help="base directory for output library files",
	)

	parser.add_argument(
		"--model_path_relative",
		dest="model_path_relative",
		action="store_true",
		help="use --model_path_relative if you want the 3D model to be linked to the footprint using relative instead of absolute path, default is absolute",
	)

	parser.add_argument(
		"--no_footprint",
		dest="footprint_creation",
		action="store_false",
		help="use --no_footprint if you do not want to create the footprint",
	)

	parser.add_argument(
		"--no_schematic",
		dest="schematic_creation",
		action="store_false",
		help="use --no_schematic if you do not want to create the schematic",
	)

	parser.add_argument(
		"-schematic_lib",
		dest="schematic_lib",
		type=str,
		default="default_lib",
		help='set schematic library name, default is "default_lib"',
	)

	parser.add_argument(
		"-footprint_lib",
		dest="footprint_lib",
		type=str,
		default="footprint",
		help='set footprint library name,  default is "footprint"',
	)

	parser.add_argument(
		"-logging_level",
		dest="logging_level",
		type=str,
		default="INFO",
		choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
		help="set logging level. If DEBUG is used, the debug logs are only written in the log file if the option  --log_file is set ",
	)

	parser.add_argument(
		"--log_file",
		dest="log_file",
		action="store_true",
		help="use --log_file if you want logs to be written in a file",
	)

	args = parser.parse_args()

	helper.set_logging(args.logging_level, args.log_file)

	for component in args.components:
		add_component(component, args)


if __name__ == "__main__":
	main()
